@@ -2,9 +2,12 @@
 
 from pyke import knowledge_engine
 
-Engine = knowledge_engine.engine('Test.CanNotProve')
+Engine = None
 
 def test(kb, ke, arg):
+    global Engine
+    if Engine is None:
+        Engine = knowledge_engine.engine(__file__)
     Engine.reset()
     Engine.activate('rules')
     try:
@@ -20,5 +23,5 @@ def dotests():
     test('rules', 'rule2', 2)
     Engine.reset()
     Engine.activate('rules')
-    (arg,), no_plan = Engine.prove_1('facts', 'fact3', (), 1)
-    assert arg == 'hi\nthere'
+    vars, no_plan = Engine.prove_1_goal('facts.fact3($ans)')
+    assert vars['ans'] == 'hi\nthere'
