@@ -1,4 +1,4 @@
-# $Id: simple_server.py 221 2009-03-26 17:17:44Z mtnyogi $
+# $Id: simple_server.py c5bb15998025 2010-03-03 dangyogi $
 # coding=utf-8
 #
 # Copyright © 2008 Bruce Frederiksen
@@ -31,13 +31,13 @@ class RequestHandlerNoLogging(wsgiref.simple_server.WSGIRequestHandler):
 def init(trace_sql = False, db_engine = 'sqlite3'):
     if db_engine.lower() == 'sqlite3':
         import sqlite3 as db
-        import examples.sqlgen.load_sqlite3_schema as load_schema
+        import sqlgen.load_sqlite3_schema as load_schema
         db_connection = \
             db.connect(os.path.join(os.path.dirname(load_schema.__file__),
                                     'sqlite3.db'))
     elif db_engine.lower() == 'mysql':
         import MySQLdb as db
-        import examples.sqlgen.load_mysql_schema as load_schema
+        import sqlgen.load_mysql_schema as load_schema
         db_connection = db.connect(user="movie_user", passwd="user_pw",
                                    db="movie_db")
     else:
