@@ -1,4 +1,4 @@
-# $Id: condensedPrint.py 2bb500de1268 2008-09-24 mtnyogi $
+# $Id: condensedPrint.py 081917d30609 2010-03-05 mtnyogi $
 # coding=utf-8
 # 
 # Copyright © 2007-2008 Bruce Frederiksen
@@ -168,11 +168,3 @@ def printDict(dict, maxdepth,
            [item(key, dict[key], maxdepth, separator) for key in keys] + \
            [endChar]
 
-def test():
-    import doctest
-    import sys
-    sys.exit(doctest.testmod()[0])
-
-if __name__ == "__main__":
-    test()
-
