@@ -1,4 +1,4 @@
-# $Id: helpers.py 172 2009-01-13 14:33:06Z mtnyogi $
+# $Id: helpers.py 081917d30609 2010-03-05 mtnyogi $
 # coding=utf-8
 # 
 # Copyright © 2007-2008 Bruce Frederiksen
@@ -30,7 +30,8 @@ def fc_head(rb_name):
         "from __future__ import with_statement",
         "from pyke import contexts, pattern, fc_rule, knowledge_base",
         "",
-        "version = %s" % repr(pyke.version)
+        "pyke_version = %r" % pyke.version,
+        "compiler_version = %r" % pyke.compiler_version,
     )
 
 def bc_head(rb_name):
@@ -41,20 +42,21 @@ def bc_head(rb_name):
         "import itertools",
         "from pyke import contexts, pattern, bc_rule",
         "",
-        "version = %s" % repr(pyke.version)
+        "pyke_version = %r" % pyke.version,
+        "compiler_version = %r" % pyke.compiler_version,
     )
 
 def plan_head(rb_name):
     return (
         "# %s_plans.py" % rb_name,
         "",
-        "version = %s" % repr(pyke.version)
+        "pyke_version = %r" % pyke.version,
+        "compiler_version = %r" % pyke.compiler_version,
     )
 
-def goal(rb_name, rule_name,
-         (goal, goal_name, pattern_args, taking, start_lineno, end_lineno),
-         pred_plan_lines, python_lines):
+def goal(rb_name, rule_name, goal_info, pred_plan_lines, python_lines):
     # returns plan_lines, goal_fn_head, goal_fn_tail, goal_decl_lines
+    goal, goal_name, pattern_args, taking, start_lineno, end_lineno = goal_info
     assert goal == 'goal'
     goal_fn_head = (
         "",
@@ -240,10 +242,3 @@ def merge_patterns(patterns, pattern_list):
 def syntax_error(msg, lineno, pos):
     raise SyntaxError(msg, scanner.syntaxerror_params(pos, lineno))
 
-def test():
-    import doctest
-    import sys
-    sys.exit(doctest.testmod()[0])
-
-if __name__ == "__main__":
-    test()
