@@ -1,4 +1,4 @@
-# $Id: kqb_parser.py 208 2009-03-25 20:17:03Z mtnyogi $
+# $Id: kqb_parser.py 49507964ae64 2010-03-27 mtnyogi $
 # coding=utf-8
 # 
 # Copyright © 2008 Bruce Frederiksen
@@ -53,6 +53,7 @@ class kqb_parser(object):
             (?P<colon> : ) 
         ) [ \t\f\r\v]* (?: \#.* )? ''', re.UNICODE | re.VERBOSE)
     pushed_token = None
+
     def __init__(self, f):
         # f needs readline() and name.
         self.f = f
@@ -60,6 +61,7 @@ class kqb_parser(object):
         self.line = ''
         self.column = 0
         self.eof = False
+
     def readline(self):
         r'''
             >>> from StringIO import StringIO
@@ -96,6 +98,7 @@ class kqb_parser(object):
                 self.indent, self.column = scanner.count_indent(line)
                 self.line = line
                 break
+
     def SyntaxError(self, msg, last_token=True):
         if last_token:
             raise SyntaxError(msg,
@@ -104,6 +107,7 @@ class kqb_parser(object):
         raise SyntaxError(msg,
                           (self.f.name, self.lineno, self.column + 1,
                            self.line))
+
     def push_token(self):
         #print "push_token:", self.last_token  # FIX
         self.pushed_token = self.last_token
@@ -111,6 +115,7 @@ class kqb_parser(object):
         self.pushed_column = self.column
         self.indent = self.last_indent
         self.column = self.last_column
+
     def get_token(self, check_token=None):
         r'''
             >>> from StringIO import StringIO
@@ -186,7 +191,7 @@ class kqb_parser(object):
         self.indent += indent
         self.column = end
         if token == 'str' or token == 'prompt':
-            value = scanner.unquote(chars)
+            value = scanner.unescape(chars)
         elif token == 'const':
             value = eval(chars)
         elif token == 'number':
@@ -211,6 +216,7 @@ class kqb_parser(object):
         self.last_token = str(token), value
         #print "get_token: returning", self.last_token  # FIX
         return self.last_token
+
     def get_block_string(self, stop=None, hanging=False, ending_newlines=False):
         r'''
             >>> from StringIO import StringIO
@@ -273,7 +279,8 @@ class kqb_parser(object):
                 for i in range(self.lineno - last_lineno - 1): ans.append('')
             ans.append(' ' * (self.indent - indent) + self.line[self.column:])
         if not ans: self.SyntaxError("expected block string", False)
-        return u'\n'.join(scanner.unquote(str) for str in ans)
+        return u'\n'.join(scanner.unescape(str) for str in ans)
+
     def parse_simple_match(self):
         token, value = self.get_token()
         if token == 'str' or token == 'id' or token == 'number' or \
@@ -307,6 +314,7 @@ class kqb_parser(object):
             next_token, next_value = self.get_token('number')
             return slice(None, next_value)
         self.SyntaxError("expected match, got %s" % token)
+
     def parse_match(self):
         r'''
             >>> from StringIO import StringIO
@@ -343,11 +351,13 @@ class kqb_parser(object):
         self.push_token()
         if len(ans) == 1: return ans[0]
         return tuple(ans)
+
     def get_value(self):
         token, value = self.get_token()
         if token not in ('const', 'number', 'id', 'str'):
             self.SyntaxError("expected value, got %s" % token)
         return value
+
     def skip_spaces(self, pre_increment=0):
         if pre_increment:
             indent, chars = \
@@ -358,6 +368,7 @@ class kqb_parser(object):
         indent, chars = scanner.count_indent(self.line[self.column:])
         self.indent += indent
         self.column += chars
+
     def parse_alternatives(self):
         r'''
             >>> from StringIO import StringIO
@@ -422,6 +433,7 @@ class kqb_parser(object):
                       if isinstance(value, tuple)) \
                   if review \
                   else None
+
     def parse_review(self):
         r'''
             >>> from StringIO import StringIO
@@ -462,6 +474,7 @@ class kqb_parser(object):
             self.SyntaxError("unexpected indent", False)
         #print "parse_review:", tuple(review)   # FIX
         return tuple(review)
+
     def parse_questions(self):
         r''' question_base.question generator.
 
@@ -520,8 +533,7 @@ class kqb_parser(object):
             if self.column >= len(self.line): self.readline()
 
 def parse_kqb(filename):
-    dirs, base = os.path.split(filename)
-    name = base[:-4]
+    name = os.path.basename(filename)[:-4]
     with open(filename, 'rU') as f:
         base = question_base.question_base(name)
         parser = kqb_parser(f)
@@ -529,10 +541,3 @@ def parse_kqb(filename):
             base.add_question(question)
     return base
 
-def test():
-    import doctest
-    import sys
-    sys.exit(doctest.testmod()[0])
-
-if __name__ == "__main__":
-    test()
