@@ -1,4 +1,4 @@
-# $Id: scanner.py 87e5bee9250a 2009-10-24 mtnyogi $
+# $Id: scanner.py 9f7068449a4b 2010-03-08 mtnyogi $
 # coding=utf-8
 # 
 # Copyright © 2007-2008 Bruce Frederiksen
@@ -32,6 +32,7 @@ from pyke.krb_compiler.ply import lex
 debug=0
 
 kfb_mode = False
+goal_mode = False
 
 states = (
     ('indent', 'exclusive'),
@@ -315,28 +316,28 @@ def t_code_NL_TOK(t):
 # strings:
 def t_tsqstring(t):
     r"[uU]?[rR]?'''([^\\]|\\.)*?'''"
-    #t.value = unquote(t.value[3:-3])
+    #t.value = unescape(t.value[3:-3])
     t.type = 'STRING_TOK'
     t.lexer.lineno += t.value.count('\n')
     return t
 
 def t_tdqstring(t):
     r'[uU]?[rR]?"""([^\\]|\\.)*?"""'
-    #t.value = unquote(t.value[3:-3])
+    #t.value = unescape(t.value[3:-3])
     t.type = 'STRING_TOK'
     t.lexer.lineno += t.value.count('\n')
     return t
 
 def t_sqstring(t):
     r"[uU]?[rR]?'([^'\\\n\r]|\\.|\\(\r)?\n)*?'"
-    #t.value = unquote(t.value[1:-1])
+    #t.value = unescape(t.value[1:-1])
     t.lexer.lineno += t.value.count('\n')
     t.type = 'STRING_TOK'
     return t
 
 def t_dqstring(t):
     r'[uU]?[rR]?"([^"\\\n\r]|\\.|\\(\r)?\n)*?"'
-    #t.value = unquote(t.value[1:-1])
+    #t.value = unescape(t.value[1:-1])
     t.type = 'STRING_TOK'
     t.lexer.lineno += t.value.count('\n')
     return t
@@ -345,13 +346,19 @@ def t_dqstring(t):
 def t_ANONYMOUS_VAR_TOK(t):
     r'\$_([a-zA-Z_][a-zA-Z0-9_]*)?'
     if kfb_mode: t_ANY_error(t)
-    t.value = "'" + t.value[1:] + "'"
+    if goal_mode:
+        t.value = t.value[1:]
+    else:
+        t.value = "'" + t.value[1:] + "'"
     return t
 
 def t_PATTERN_VAR_TOK(t):
     r'\$[a-zA-Z][a-zA-Z0-9_]*'
     if kfb_mode: t_ANY_error(t)
-    t.value = "'" + t.value[1:] + "'"
+    if goal_mode:
+        t.value = t.value[1:]
+    else:
+        t.value = "'" + t.value[1:] + "'"
     return t
 
 def t_IDENTIFIER_TOK(t):
@@ -481,7 +488,7 @@ escapes = {
     '\"': '\"',
 }
 
-def unquote(s):
+def unescape(s):
     start = 0
     ans = []
     i = s.find('\\', start)
@@ -624,21 +631,27 @@ def syntaxerror_params(pos = None, lineno = None):
         argument to SyntaxError exceptions.
     '''
     if pos is None: pos = lexer.lexpos
-    start = pos
+    if pos > len(lexer.lexdata): pos = len(lexer.lexdata)
+    end = pos
     if lineno is None: lineno = lexer.lineno
-    while start > 0 and (start >= len(lexer.lexdata) or
-                         lexer.lexdata[start] in '\r\n'):
-        start -= 1
-    end = start
-    if debug: print "pos", pos, "lineno", lineno, "start", start
-    start = max(lexer.lexdata.rfind('\r', 0, start),
-                lexer.lexdata.rfind('\n', 0, start)) + 1
+    while end > 0 and (end >= len(lexer.lexdata) or
+                       lexer.lexdata[end] in '\r\n'):
+        end -= 1
+    start = end
+    if debug: print "pos", pos, "lineno", lineno, "end", end
+    start = max(lexer.lexdata.rfind('\r', 0, end),
+                lexer.lexdata.rfind('\n', 0, end)) + 1
     column = pos - start + 1
     end1 = lexer.lexdata.find('\r', end)
     end2 = lexer.lexdata.find('\n', end)
-    if end1 < 0: end = end2
+    if end1 < 0:
+        if end2 < 0: end = len(lexer.lexdata)
+        else: end = end2
     elif end2 < 0: end = end1
     else: end = min(end1, end2)
+    if goal_mode and start == 0 and lexer.lexdata.startswith('check ', start):
+        start += 6
+        column -= 6
     if debug: print "start", start, "column", column, "end", end
     return (lexer.filename, lineno, column, lexer.lexdata[start:end])
 
@@ -675,10 +688,3 @@ def init(this_module, debug_param, check_tables = False, kfb = False):
                             lextab='pyke.krb_compiler.scanner_tables',
                             outputdir=os.path.dirname(this_module.__file__))
 
-def test():
-    import doctest
-    import sys
-    sys.exit(doctest.testmod()[0])
-
-if __name__ == "__main__":
-    test()
