@@ -1,4 +1,4 @@
-# $Id: target_pkg.py bd350590fc64 2009-11-03 mtnyogi $
+# $Id: target_pkg.py b034e6155de4 2010-04-13 mtnyogi $
 # coding=utf-8
 # 
 # Copyright © 2009 Bruce Frederiksen
@@ -22,7 +22,7 @@
 # THE SOFTWARE.
 
 """
-    The target_pkg object keeps track of all of the compiled files within one
+    Each target_pkg object keeps track of all of the compiled files within one
     compiled_krb package.
 """
 
@@ -50,7 +50,8 @@ class target_pkg(object):
     This maintains the following information for each compiled target file:
         source_package, source_filepath, compile_time, target_filename.
     '''
-    def __init__(self, module_name, filename, pyke_version = pyke.version,
+    def __init__(self, module_name, filename = None,
+                       pyke_version = pyke.version,
                        loader = None, sources = None, compiler_version = 0):
         r'''
 
@@ -63,9 +64,10 @@ class target_pkg(object):
             pyke_version: the version of pyke used to compile the target files.
             loader:       the __loader__ attribute of the compiled_pyke_files
                           module (only set if the compiled_krb directory has
-                          been zipped).
-            sources:      {(source_module, source_filepath):
-                            [compile_time, target_file...]}
+                          been zipped, otherwise None).
+            sources:      {(source_package_name, path_from_package,
+                            source_filepath):
+                           [compile_time, target_file...]}
             compiler_version:
                           the version of the pyke compiler used to compile all
                           of the targets in this compiled_krb directory.
@@ -77,15 +79,81 @@ class target_pkg(object):
 
             In this case, all of the parameters are passed to __init__.
 
-        2.  From knowledge_engine.engine.__init__ (actually _init_path).
+        2.  From knowledge_engine.engine.__init__ (actually _create_target_pkg).
 
-            In this case, only the first two parameters are passed to __init__.
+            In this case, only the first parameter is passed to __init__.
 
         Either way, after importing compiled_pyke_files or creating a new
         instance directly, reset is called by
-        knowledge_engine.engine._init_path.
+        knowledge_engine.engine._create_target_pkg.
         '''
+
+        # compiled_krb package name
         self.package_name = module_name.rsplit('.', 1)[0]
+
+        if sources is None:
+            # compiled_pyke_files.py does not exist.
+
+            # Creating a new target_pkg object from scratch.
+            try:
+                # See if the self.package_name (e.g., compiled_krb) exists.
+                target_package_dir = \
+                    os.path.dirname(import_(self.package_name).__file__)
+            except ImportError:
+                if debug:
+                    print >> sys.stderr, "target_pkg: no target package", \
+                                         self.package_name
+                # Create the target_package.
+                last_dot = self.package_name.rfind('.')
+                if last_dot < 0:
+                    assert filename is not None
+                    package_parent_dir = \
+                      os.path.dirname(os.path.dirname(filename))
+                else:
+                    package_parent_dir = \
+                      os.path.dirname(
+                        # This import better work!
+                        import_(self.package_name[:last_dot]).__file__)
+                    if filename is not None:
+                        assert os.path.normpath(
+                                 os.path.abspath(package_parent_dir)) == \
+                               os.path.normpath(
+                                 os.path.dirname(os.path.dirname(filename))), \
+                          "Internal error: %r != %r" % (
+                            os.path.normpath(
+                              os.path.abspath(package_parent_dir)),
+                            os.path.normpath(
+                              os.path.dirname(os.path.dirname(filename))))
+                if debug:
+                    print >> sys.stderr, "target_pkg package_parent_dir:", \
+                                         package_parent_dir
+                target_package_dir = \
+                    os.path.join(package_parent_dir,
+                                 self.package_name[last_dot + 1:])
+                if debug:
+                    print >> sys.stderr, "target_pkg target_package_dir:", \
+                                         target_package_dir
+                if not os.path.lexists(target_package_dir):
+                    if debug:
+                        print >> sys.stderr, "target_pkg: mkdir", \
+                                             target_package_dir
+                    os.mkdir(target_package_dir)
+
+                # Does __init__.py file exist?
+                init_filepath = \
+                    os.path.join(target_package_dir, '__init__.py')
+                if debug:
+                    print >> sys.stderr, "target_pkg init_filepath:", \
+                                         init_filepath
+                if not os.path.lexists(init_filepath):
+                    # Create empty __init__.py file.
+                    if debug:
+                        print >> sys.stderr, "target_pkg: creating", \
+                                             init_filepath
+                    open(init_filepath, 'w').close()
+            filename = os.path.join(target_package_dir,
+                                    'compiled_pyke_files.py')
+
         if filename.endswith('.py'):
             self.filename = filename
         else:
@@ -100,62 +168,81 @@ class target_pkg(object):
             #  [compile_time, target_filename, ...]}
             self.sources = sources if sources is not None else {}
         elif self.loader is None:
+            # Force recompile of everything
             self.sources = {}
         else:
-            # loading incorrect version from zip file
+            # Loading incorrect pyke.compiler_version from zip file.
+            # Can't recompile to zip file...
             raise AssertionError("%s: wrong version of pyke, "
                                  "running %s, compiled for %s" % 
                                  (module_name, pyke.version, pyke_version))
 
-    def reset(self):
+    def reset(self, check_sources = True):
         ''' This should be called once by engine.__init__ prior to calling
             add_source_package.
         '''
         if debug: print >> sys.stderr, "target_pkg.reset"
         self.dirty = False
-        self.source_packages = {}  # {source_package_name: source_package_dir}
+        self.check_sources = check_sources
+
+        # {(source_package_name, path_from_package): source_package_dir}
+        self.source_packages = {}
+
         self.compiled_targets = set()  # set of target_filename
         self.rb_names = set()
 
-    def add_source_package(self, source_package_name):
+    def add_source_package(self, source_package_name, path_from_package,
+                                 source_package_dir):
         if debug:
-            print >> sys.stderr, "target_pkg.add_source_package:", \
-                                 source_package_name
+            print >> sys.stderr, "target_pkg.add_source_package " \
+                                   "source_package_name:", \
+                                 repr(source_package_name)
+            print >> sys.stderr, "    path_from_package:", \
+                                 repr(path_from_package)
+            print >> sys.stderr, "    source_package_dir:", \
+                                 repr(source_package_dir)
         if not self.loader:
-            assert source_package_name not in self.source_packages, \
-                   "duplicate source package: %s" % source_package_name
-            source_package_dir = \
-                os.path.dirname(import_(source_package_name).__file__)
-            if debug:
-                print >> sys.stderr, "source_package_dir:", source_package_dir
-            self.source_packages[source_package_name] = source_package_dir
+            assert (source_package_name, path_from_package) not in \
+                     self.source_packages, \
+                   "duplicate source package: %s" % path_from_package
+            source_dir = os.path.normpath(os.path.join(source_package_dir,
+                                                       path_from_package))
+            self.source_packages[source_package_name, path_from_package] = \
+                source_dir
             sources = set([])
             for dirpath, dirnames, filenames \
-             in os.walk(source_package_dir, onerror=_raise_exc):
+             in os.walk(source_dir, onerror=_raise_exc):
                 for filename in filenames:
                     if filename.endswith(('.krb', '.kfb', '.kqb')):
                         source_abspath = os.path.join(dirpath, filename)
-                        assert dirpath.startswith(source_package_dir)
+                        assert dirpath.startswith(source_dir)
                         source_relpath = \
-                            os.path.join(dirpath[len(source_package_dir)+1:],
+                            os.path.join(dirpath[len(source_dir)+1:],
                                          filename)
-                        self.add_source(source_package_name, source_relpath,
+                        self.add_source(source_package_name, path_from_package,
+                                        source_relpath,
                                         os.path.getmtime(source_abspath))
                         sources.add(source_relpath)
-            # delete any old source files that are no longer present
-            for filepath in [src_filepath
-                             for src_pkg_name, src_filepath
-                              in self.sources.iterkeys()
-                                 if src_pkg_name == source_package_name and \
-                                    src_filepath not in sources]:
+
+            # Delete old source file info for files that are no longer present
+            for deleted_filepath \
+             in [src_filepath
+                 for src_pkg_name, src_path_from_pkg, src_filepath
+                  in self.sources.iterkeys()
+                     if src_pkg_name == source_package_name and
+                        src_path_from_pkg == path_from_package and
+                        src_filepath not in sources]:
                 if debug:
                     print >> sys.stderr, "del:", source_package_name, filepath
-                del self.sources[source_package_name, filepath]
+                del self.sources[source_package_name, path_from_package,
+                                 deleted_filepath]
 
-    def add_source(self, source_package_name, source_filepath, source_mtime):
+    def add_source(self, source_package_name, path_from_package,
+                         source_filepath, source_mtime):
         if debug:
             print >> sys.stderr, "target_pkg.add_source:", \
-                                 source_package_name, source_filepath
+                                 source_package_name, path_from_package, \
+                                 source_filepath
         rb_name = os.path.splitext(os.path.basename(source_filepath))[0]
         if debug: print >> sys.stderr, "rb_name:", rb_name
         if not Name_test.match(rb_name):
@@ -164,7 +251,7 @@ class target_pkg(object):
         if rb_name in self.rb_names:
             raise ValueError("%s: duplicate knowledge base name" % rb_name)
         self.rb_names.add(rb_name)
-        key = source_package_name, source_filepath
+        key = source_package_name, path_from_package, source_filepath
         if debug: print >> sys.stderr, "key:", key
         if self.sources.get(key, (0,))[0] < source_mtime:
             if debug:
@@ -179,11 +266,14 @@ class target_pkg(object):
     def compile(self, engine):
         if debug: print >> sys.stderr, "%s.compile:" % self.package_name
         global krb_compiler
-        if not self.loader:
+        if self.check_sources and not self.loader:
             initialized = False
-            for (source_package_name, source_filename), value \
+            for (source_package_name, path_from_package, source_filename), \
+                value \
              in self.sources.iteritems():
-                if not value and source_package_name in self.source_packages:
+                if not value and \
+                   (source_package_name, path_from_package) in \
+                     self.source_packages:
                     if not initialized:
                         try:
                             krb_compiler
@@ -193,7 +283,8 @@ class target_pkg(object):
                     target_files = \
                         self.do_by_ext('compile',
                             os.path.join(
-                                self.source_packages[source_package_name],
+                                self.source_packages[source_package_name,
+                                                     path_from_package],
                                 source_filename))
                     if debug: print >> sys.stderr, "target_files:", target_files
                     value.append(time.time())
@@ -237,29 +328,38 @@ class target_pkg(object):
             with open(self.filename, 'w') as f:
                 f.write("# compiled_pyke_files.py\n\n")
                 f.write("from pyke import target_pkg\n\n")
-                f.write("pyke_version = %r\n\n" % pyke.version)
-                f.write("compiler_version = %r\n\n" % pyke.compiler_version)
-                f.write("try:\n");
+                f.write("pyke_version = %r\n" % pyke.version)
+                f.write("compiler_version = %r\n" % pyke.compiler_version)
+                f.write("target_pkg_version = %r\n\n" % pyke.target_pkg_version)
+                f.write("try:\n")
                 f.write("    loader = __loader__\n")
-                f.write("except NameError:\n");
-                f.write("    loader = None\n\n");
-                f.write("targets = target_pkg.target_pkg(__name__, __file__, "
+                f.write("except NameError:\n")
+                f.write("    loader = None\n\n")
+                f.write("def get_target_pkg():\n")
+                f.write("    return target_pkg.target_pkg(__name__, __file__, "
                         "pyke_version, loader, {\n")
-                for item in self.sources.iteritems():
-                    if debug: print >> sys.stderr, "write got:", item
-                    if item[0][0] in self.source_packages:
-                        if debug: print >> sys.stderr, "writing:", item
-                        f.write("    %r: %r,\n" % item)
-                f.write("  },\n  compiler_version)\n")
+                for key, value in self.sources.iteritems():
+                    if debug: print >> sys.stderr, "write got:", key, value
+                    if (key[0], key[1]) in self.source_packages:
+                        if debug: print >> sys.stderr, "writing:", key, value
+                        f.write("         %r:\n" % (key,))
+                        f.write("           %r,\n" % (value,))
+                f.write("        },\n")
+                f.write("        compiler_version)\n\n")
+            if os.path.exists(self.filename + 'c'):
+                os.remove(self.filename + 'c')
+            if os.path.exists(self.filename + 'o'):
+                os.remove(self.filename + 'o')
 
     def load(self, engine, load_fc = True, load_bc = True,
                            load_fb = True, load_qb = True):
         load_flags = {'load_fc': load_fc, 'load_bc': load_bc,
                       'load_fb': load_fb, 'load_qb': load_qb}
         if debug: print >> sys.stderr, "target_pkg.load:", load_flags
-        for (source_package_name, source_filename), value \
+        for (source_package_name, path_from_package, source_filename), value \
          in self.sources.iteritems():
-            if self.loader or source_package_name in self.source_packages:
+            if not self.check_sources or self.loader or \
+               (source_package_name, path_from_package) in self.source_packages:
                 for target_filename in value[1:]:
                     if debug: print >> sys.stderr, "load:", target_filename
                     self.do_by_ext('load', target_filename, engine, load_flags)
