@@ -1,4 +1,4 @@
-# $Id: test.py 237 2009-04-21 18:05:08Z mtnyogi $
+# $Id: test.py 081917d30609 2010-03-05 mtnyogi $
 # coding=utf-8
 #
 # Copyright © 2008 Bruce Frederiksen
@@ -258,11 +258,3 @@ def run(rule_bases_to_activate,
         except:
             krb_traceback.print_exc(100)
 
-def test():
-    import doctest
-    import sys
-    sys.exit(doctest.testmod()[0])
-
-if __name__ == "__main__":
-    test()
-
