@@ -1,4 +1,4 @@
-# $Id: unique.py 664ff001a2ca 2007-11-08 mtnyogi $
+# $Id: unique.py 081917d30609 2010-03-05 mtnyogi $
 # coding=utf-8
 # 
 # Copyright © 2007 Bruce Frederiksen
@@ -23,12 +23,6 @@
 
 class unique(object):
     def __init__(self, name): self.name = name
+
     def __repr__(self): return "<unique %s>" % self.name
 
-def test():
-    import doctest
-    import sys
-    sys.exit(doctest.testmod()[0])
-
-if __name__ == "__main__":
-    test()
